<?php declare(strict_types=1);

/*
 * Plugin Name: Most Read - Google Analytics
 * Description: A list of articles fetched from Google Analytics. Has option to add headline, link to section and provide custom view id.
 * Version: 1.0.0
 * Author: Infomaker Scandinavia AB
 * Author URI: https://infomaker.se
 */

namespace Everyware\Plugin\GoogleAnalytics;

use Everyware\ProjectPlugin\Components\Adapters\WidgetAdapter;
use Everyware\ProjectPlugin\Components\ComponentSettingsRepository;
use Everyware\ProjectPlugin\Components\Contracts\Admin;
use Everyware\ProjectPlugin\Components\Contracts\InfoManager;
use Everyware\ProjectPlugin\Components\SettingsProviders\SimpleSettingsProvider;
use Everyware\ProjectPlugin\Components\WidgetAdmin;
use Infomaker\Everyware\Support\Str;
use Everyware\ProjectPlugin\Helpers\FileReader;
use Infomaker\Everyware\Twig\View;
use Infomaker\Everyware\Support\Environment;
use OcArticle;

class MostReadWidget extends WidgetAdapter
{
    /**
     * @var array
     */
    protected static $fields = [
        'custom_view_id' => '',
        'title' => '',
        'limit' => 1,
        'render_as_list' => false,
        'page' => '',
        'start' => 1,
        'template' => '',
    ];

    /**
     * @var InfoManager
     */
    protected static $infoManager;

    /**
     * @var GoogleAnalyticsClient
     */
    protected static $client;

    /**
     * Twig-template to use
     *
     * @var string
     */
    protected $templatePath = '@plugins/widgets/most-read';

    /** Contains the default template to use when rendering articles in the list
     *
     * @var string
     */
    private $defaultTemplate = 'ocarticle-default.php';

    protected function widgetSetup(): Admin
    {
        add_action(
            'admin_enqueue_scripts',
            function () {
                $assetsPath = plugin_dir_url(__FILE__).'../assets';
                $scriptVersion = Environment::isDev() ? false : GIT_COMMIT;
                wp_enqueue_script(
                    'everyware-google-analytics-widget-admin',
                    "{$assetsPath}/js/widget-admin.js",
                    ['jquery'],
                    $scriptVersion,
                    true
                );
            }
        );

        return new WidgetAdmin(
            new MostReadForm(static::$infoManager ?? new FileReader(__FILE__)),
            new ComponentSettingsRepository(new SimpleSettingsProvider(static::$fields))
        );
    }

    public static function setInfoManager(InfoManager $infoManager): void
    {
        static::$infoManager = $infoManager;
    }

    public static function setClient(GoogleAnalyticsClient $client): void
    {
        static::$client = $client;
    }

    /**
     * Run the article through the template to generate HTML
     *
     * @param OcArticle[] $ocArticles
     * @param string       $template
     *
     * @return array
     */
    private function generateFromTemplate($ocArticles, $template = ''): array
    {
        $articles = [];
        ob_start();
        foreach ($ocArticles as $index => $article) {
            include $template;
            $articles[] = ob_get_contents();
            ob_clean();
        }
        ob_end_clean();

        return $articles;
    }

    private function getTemplate($template = '')
    {
        $templateFile = Str::notEmpty($template) ? $template : $this->defaultTemplate;

        return locate_template("templates/{$templateFile}");
    }

    /**
     * @param array $viewData
     * @param array $args
     *
     * @return string
     */
    protected function generateWidget(array $viewData, array $args): string
    {
        if ($viewData['custom_view_id'] !== '') {
            static::$client->customize(
                ['view_id' => $viewData['custom_view_id']]
            );
        }

        // Get real article data back from WP and OC
        $articles = static::$client->getArticlesFromPosition($viewData['start'], $viewData['limit']);

        return View::generate("{$this->templatePath}/widget", array_replace($viewData, [
            'articles' => $this->generateFromTemplate($articles, $this->getTemplate($viewData['template'])),
            'args' => $args,
            'settings' => $this->getWidgetSettings()
        ]));
    }
}
